from dataclasses import dataclass
from socket import socket
from datetime import datetime
from functools import reduce
from typing import List, Callable, Tuple 
from .method import Method
from .response import Response
from .responsecodes import ResponseCode
from .content import *
from .patchers import patchers
from .logger import log
import os
import traceback

@dataclass
class Route: 
    matcher: Callable 
    methods: List[Method]
    handler: Callable[['Request', socket, Tuple[str, int]], Response]

    def method_is_allowed(self, method: Method) -> bool: 
        return method in self.methods 

    def execute(self, *args): 
        try: 
            response = self.handler(*args)
            for patcher in patchers:
                response = patcher(response, args[0])

            return response

        except Exception as e:
            log.error(traceback.format_exc())
            return error_page(500)

    def matches(self, request: 'Request') -> bool:
        if not self.method_is_allowed(request.method): return False
        return self.matcher(request.path)

routes = [
    Route(
        lambda request: request.path == '/',
        [Method.GET],
        lambda request, *_: Response(
            ResponseCode.OK, 
            {'Content-Type': 'text/html'},
            parse_file('./index.html').encode('utf-8')
        )
    ),
    Route(
        lambda request: os.path.isfile('.' + request.path) and request.path.endswith('.html'),
        [Method.GET],
        lambda request, *_: Response(
            ResponseCode.OK,
            {'Content-Type': 'text/html'},
            parse_file('.' + request.path.path).encode('utf-8')
        )
    ),
    Route(
        lambda request: os.path.isfile('.' + request.path),
        [Method.GET],
        lambda request, *_: Response(
            ResponseCode.OK, 
            *raw_file_contents('.' + request.path.path)
        )
    ),
    Route(
        lambda _: True, 
        [Method.GET],
        lambda *_: error_page(404)
    )
]
